"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _rootNode;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Node = void 0;
class Node {
    constructor() {
        this.map = new Map();
        this.value = null;
    }
}
exports.Node = Node;
var ResultValue;
(function (ResultValue) {
    ResultValue[ResultValue["notFound"] = 0] = "notFound";
    ResultValue[ResultValue["partiallyFound"] = 1] = "partiallyFound";
    ResultValue[ResultValue["found"] = 2] = "found";
})(ResultValue || (ResultValue = {}));
class Trie {
    constructor() {
        _rootNode.set(this, new Node());
        this.logging = false;
    }
    set(keys, val) {
        this.log(`Calling set with keys "${JSON.stringify(keys)}", val: "${JSON.stringify(val)}"`);
        this.checkKeys(keys);
        let node = __classPrivateFieldGet(this, _rootNode);
        for (const key of keys) {
            this.log(`Setting key "${key}"`);
            if (this.isKeyWildcard(key)) {
                if (!node.wildcardMap) {
                    node.wildcardMap = new Map();
                }
                this.log(`Setting wildcard at key "${key}"`);
                const newNode = new Node();
                node.wildcardMap.set(key, newNode);
                node = newNode;
            }
            else {
                const target = node.map.get(key);
                if (target) {
                    this.log('Target found');
                    node = target;
                }
                else {
                    this.log('Creating new node');
                    const newNode = new Node();
                    node.map.set(key, newNode);
                    node = newNode;
                }
            }
        }
        node.value = val;
    }
    getWithPayload(keys) {
        this.log(`Calling getWithPayload with keys "${JSON.stringify(keys)}"`);
        this.checkKeys(keys);
        return this.getWithPayloadCore(__classPrivateFieldGet(this, _rootNode), keys, 0, undefined);
    }
    get(keys) {
        const [result] = this.getWithPayload(keys);
        return result;
    }
    getWithPayloadCore(node, keys, index, inheritedPayload) {
        if (index >= keys.length) {
            this.log(`Found the value "${node.value}", payload ${JSON.stringify(inheritedPayload)}`);
            return [node.value, inheritedPayload];
        }
        const key = keys[index];
        this.logWithKey(key, `Getting value with key "${key}"`);
        let firstAttemptResult = [
            undefined,
            undefined,
        ];
        const target = node.map.get(key);
        if (target) {
            this.logWithKey(key, 'Target found');
            firstAttemptResult = this.getWithPayloadCore(target, keys, index + 1, inheritedPayload);
        }
        this.logWithKey(key, `Checking first attempt result "${JSON.stringify(firstAttemptResult[0])}"`);
        if (firstAttemptResult[0] !== undefined && firstAttemptResult[0] !== null) {
            this.logWithKey(key, `Found the value at first attempt "${firstAttemptResult[0]}"`);
            return firstAttemptResult;
        }
        const { wildcardMap } = node;
        if (wildcardMap) {
            let secondAttemptResult = [
                undefined,
                undefined,
            ];
            for (const [nodeKey, nodeValue] of wildcardMap.entries()) {
                this.logWithKey(key, `Keep searching with child entry "${nodeKey}" in wildcard mode`);
                const payload = {
                    ...inheritedPayload,
                    ...this.getWildcardPayload(key, nodeKey, nodeValue.value),
                };
                this.logWithKey(key, `Handling wildcard, key "${key}", value "${node.value}" payload ${JSON.stringify(payload)}`);
                const res = this.getWithPayloadCore(nodeValue, keys, index + 1, payload);
                const resValue = this.resultValue(res[0]);
                this.logWithKey(key, `Checking second attempt result "${JSON.stringify(secondAttemptResult[0])}"`);
                if (resValue > this.resultValue(secondAttemptResult[0])) {
                    secondAttemptResult = res;
                    if (resValue >= ResultValue.found) {
                        return res;
                    }
                }
            }
            this.logWithKey(key, 'Nothing found in wildcard mode');
            return secondAttemptResult;
        }
        this.logWithKey(key, 'Nothing found');
        return firstAttemptResult;
    }
    isKeyWildcard(_key) {
        return false;
    }
    getWildcardPayload(_key, _wildcardName, _wildcardValue) {
        return undefined;
    }
    checkKeys(keys) {
        if (!(keys === null || keys === void 0 ? void 0 : keys.length)) {
            throw new Error(`Empty keys, got "${JSON.stringify(keys)}"`);
        }
    }
    log(s) {
        if (this.logging) {
            console.log(s);
        }
    }
    logWithKey(key, msg) {
        this.log(`KEY: ${key} | ${msg}`);
    }
    resultValue(result) {
        if (result === undefined) {
            return ResultValue.notFound;
        }
        if (result === null) {
            return ResultValue.partiallyFound;
        }
        return ResultValue.found;
    }
}
exports.default = Trie;
_rootNode = new WeakMap();
//# sourceMappingURL=main.js.map